/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.monitor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tinygroup.cepcore.CEPCore;
import org.tinygroup.cepcore.impl.AbstractEventProcessor;
import org.tinygroup.event.Event;
import org.tinygroup.event.ServiceInfo;
import org.tinygroup.event.ServiceRequest;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.monitor.config.MonitorProcessor;
import org.tinygroup.monitor.config.MonitorProcessors;

/**
 * 抽象监控器
 * 
 * @author luoguo
 * 
 */
public abstract class AbstractMonitor extends AbstractEventProcessor {
	private static Logger logger = LoggerFactory
			.getLogger(AbstractMonitor.class);
	private CEPCore cepCore;
	private Object monitorObject;
	private MonitorProcessors monitorProcessors;
	List<ServiceInfo> serviceInfos = new ArrayList<ServiceInfo>();
	private Map<MonitorProcessor, MonitorProcessorInterface<Object, Object>> monitorProcessorMap = new HashMap<MonitorProcessor, MonitorProcessorInterface<Object, Object>>();

	public void setCepCore(CEPCore cepCore) {
		this.cepCore = cepCore;
	}

	public CEPCore getCepCore() {
		return cepCore;
	}

	public Object getMonitorObject() {
		return monitorObject;
	}

	public void setMonitorObject(Object monitorObject) {
		this.monitorObject = monitorObject;
	}

	public MonitorProcessors getMonitorProcessors() {
		return monitorProcessors;
	}

	public void setMonitorProcessors(MonitorProcessors monitorProcessors) {
		this.monitorProcessors = monitorProcessors;
		initProcessorMap();
		initServiceInfos();
	}

	private void initServiceInfos() {
		serviceInfos.clear();
		String artId = monitorProcessors.getArtifactId();
		String groupId = monitorProcessors.getGroupId();
		for (MonitorProcessor processor : monitorProcessors
				.getMonitorProcessors()) {
			String serviceName = String.format("%s.%s.%s", groupId, artId,
					processor.getName());
			serviceInfos.add(new MonitorServiceInfo(processor.getId(),
					serviceName));
		}
	}

	private void initProcessorMap() {
		monitorProcessorMap.clear();
		for (MonitorProcessor monitorProcessor : monitorProcessors
				.getMonitorProcessors()) {
			try {// 创建实例
				monitorProcessorMap.put(monitorProcessor,
						getInstance(monitorProcessor.getClassName()));
			} catch (Exception e) {
				logger.logMessage(
						LogLevel.ERROR,
						"创建MonitorProcessor实例时出错,ClassName:"
								+ monitorProcessor.getClassName());
			}
		}
	}

	@SuppressWarnings("unchecked")
	private MonitorProcessorInterface<Object, Object> getInstance(
			String className) throws InstantiationException,
			IllegalAccessException, ClassNotFoundException {
		return  (MonitorProcessorInterface<Object, Object>) this.getClass()
				.getClassLoader().loadClass(className).newInstance();
	}

	public void process(Event event) {
		logger.logMessage(LogLevel.DEBUG, "Monitor开始处理事件,[EventId:{0}]",
				event.getEventId());
		ServiceRequest serviceRequest = event.getServiceRequest();
		boolean sameGroupId = serviceRequest.getGroupId().equals(
				monitorProcessors.getGroupId());
		boolean sameArtifactId = serviceRequest.getArtifactId().equals(
				monitorProcessors.getArtifactId());
		boolean sameGroupArtifact = sameArtifactId && sameGroupId;

		if (monitorProcessors != null
				&& monitorProcessors.getMonitorProcessors() != null) {
			for (MonitorProcessor monitorProcessor : monitorProcessors
					.getMonitorProcessors()) {
				boolean sameId = serviceRequest.getServiceId().equals(
						monitorProcessor.getId());
				boolean sameVersion = serviceRequest.getVersion().equals(
						monitorProcessor.getVersion());
				boolean sameName = serviceRequest.getName().equals(
						monitorProcessor.getName());
				if (sameId || (sameGroupArtifact && sameName)) {
					if (serviceRequest.getVersion() == null || sameVersion) {
						deal(monitorProcessor, event);
						return;
					}
				}
			}
		}
		logger.logMessage(LogLevel.DEBUG,
				"Monitor处理事件完成,未找到合适的MonitorProcessor,[EventId:{0}]",
				event.getEventId());
	}

	private void deal(MonitorProcessor monitorProcessor, Event event) {
		MonitorProcessorInterface<Object, Object> mp = monitorProcessorMap
				.get(monitorProcessor);
		logger.logMessage(LogLevel.DEBUG,
				"已找到合适的MonitorProcessor,[EventId:{0}]", event.getEventId());
		event.getServiceRequest().getContext()
				.put("result", mp.process(monitorObject));

		logger.logMessage(LogLevel.DEBUG, "Monitor处理事件完成,[EventId:{0}]",
				event.getEventId());
	}

	public List<ServiceInfo> getServiceInfos() {
		return serviceInfos;
	}
}
